@@ -58,16 +58,29 @@ else:
                          gconf.VALUE_INT: gconf.Value.get_int,
                          gconf.VALUE_STRING: gconf.Value.get_string}
                          
-    _gconfSetFunction = {bool: _config.set_bool,
-                         float: _config.set_float,
-                         int: _config.set_int,
-                         str: _config.set_string}
+    _gconfSetFunction = {bool:    _config.set_bool,
+                         float:   _config.set_float,
+                         int:     _config.set_int,
+                         str:     _config.set_string,
+                         unicode: _config.set_string}
               
 # Config default values
-_defaults = {'show_toolbar': True,
-             'show_history': True,
-             'fullscreen':   False,
-             'show_3d':      False}
+_defaults = {'show_toolbar':                     True,
+             'show_history':                     True,
+             'maximised':                        False,
+             'fullscreen':                       False,
+             'show_3d':                          False,
+             'show_move_hints':                  True,
+             'move_format':                      'human',
+             'promotion_type':                   'queen',
+             'board_view':                       'human',
+             'load_directory':                   '',
+             'save_directory':                   '',
+             'new_game_dialog/move_time':        0,
+             'new_game_dialog/white/type':       '',
+             'new_game_dialog/white/difficulty': '',
+             'new_game_dialog/black/type':       '',
+             'new_game_dialog/black/difficulty': ''}
 
 class Error(Exception):
     """Exception for configuration use"""
@@ -140,6 +153,9 @@ def set(name, value):
                 
         # Save configuration
         _save()
+        
+def default(name):
+    set(name, _defaults[name])
                 
 def _watch(client, _, entry, (function, name)):
     value = get(name)
@@ -175,7 +191,7 @@ def _save():
     root = document.createElement('config')
     document.appendChild(root)
     
-    valueNames = {int: 'int', bool: 'bool', float: 'float', str: 'str'}
+    valueNames = {int: 'int', bool: 'bool', float: 'float', str: 'str', unicode: 'str'}
 
     names = _values.keys()
     names.sort()
