@@ -173,11 +173,9 @@ class UI (gconf_wrapper.GConfWrapper):
 
         self.cleared = [] # used for Undo memory
         self.cleared_notes = [] # used for Undo memory
-        gnome.program_init('gnome-sudoku',VERSION,
-                           properties={gnome.PARAM_APP_DATADIR:APP_DATA_DIR}
-                           )
+        gnome.program_init('gnome-sudoku',VERSION, properties={gnome.PARAM_APP_DATADIR:APP_DATA_DIR})
         self.w.set_default_size(self.gconf['width'], self.gconf['height'])
-        self.w.set_title(APPNAME);
+        self.w.set_title(APPNAME_SHORT)
         self.w.connect('configure-event',self.resize_cb)
         self.w.connect('delete-event',self.quit_cb)
         self.vb = gtk.VBox()
@@ -822,7 +820,8 @@ class UI (gconf_wrapper.GConfWrapper):
         try:
             gnome.help_display('gnome-sudoku')
         except gobject.GError, e:
-            print 'Unable to display help: %s' % str(e)
+            # FIXME: This should create a pop-up dialog
+            print _('Unable to display help: %s') % str(e)
 
     @simple_debug
     def print_game (self, *args):
